# encoding: UTF-8


# The file names reflect the tweets classes names.
# All classes are considered mutually exclusive, but there are 2 classes that have intersection of subjects.
# Those are: cybersecurity_threats_general_security_news and general_security_news_cybersecurity_news
# The difference between them is their type of vector. In the first class a cybersecurity agent (person,
# entity, etc) give birth to their news while the second one has a general security agent. 

# The classified_tweets_last_run class only store the tweets classified in the last time this script was run.

# To run the script you must type as follows: ruby classify-tweets.rb tweets_security_dataset.json
# tweets_security_dataset.json is the file that contains the tweets to classify
# The json class files are stored in the 'dataset' folder as you can see in the code below


require 'rubygems'
require 'json'

`mkdir ../dataset` unless system("test -d ../dataset")

FILE_FOLDER = `pwd`.gsub(/[^\/]*\n/, '') + "dataset"

#file with tweets to classify
tweets = IO.readlines(ARGV[0])
LINES = tweets.size

# hashes to file names and file objects
  
classes = {
  :potential_cybersecurity_threats => "#{FILE_FOLDER}/potential_cybersecurity_threats.json",
  :cybersecurity_threats => "#{FILE_FOLDER}/cybersecurity_threats.json",
  :cybersecurity_threats_general_security_news => "#{FILE_FOLDER}/cybersecurity_threats_general_security_news.json",
  :general_security_news_cybersecurity_news => "#{FILE_FOLDER}/general_security_news_cybersecurity_news.json",
  :general_security_news => "#{FILE_FOLDER}/general_security_news.json",
  :spam => "#{FILE_FOLDER}/spam.json",
  :cybersecurity_news => "#{FILE_FOLDER}/cybersecurity_news.json",
}
last_classified_tweets = {:classified_tweets_last_run => "#{FILE_FOLDER}/classified_tweets_last_run.json"}

files = {}

# Open or create the files that contains the tweets for each class.
files[:classified_tweets_last_run] = File.new(last_classified_tweets[:classified_tweets_last_run], "w")

classes.each_key do |key|
  if File.exists?(classes[key]) 
    files[key] = File.open(classes[key], "a")
  else
    files[key] = File.new(classes[key], "w")
  end
end


instructions = "You are going to classify text that will appear on in the output.\n" +
               "For each text you must type the class number you think it belongs to.\n" +
                "1 - Cybersecurity news\n" +
                "2 - Potential Cybersecurity threat\n" +
                "3 - Cybersecurity threat\n" +
                "4 - Cybersecurity threats (affecting general security)\n" +
                "5 - General security (giving birth to cybersecurity concern)\n" +
                "6 - General Security news\n" +
                "7 - Spam\n" +
                "exit - finish the classifying process and finish the script\n\n"

print instructions

# Find the number of lines and sort a number to classify. Search for it in a hash.
# If the line_number is the hash sort another number, else insert the line_number
# in the hash as a key. After read the tweet and classify it.
classified_tweets = {}

srand()

i = 1
while true

  begin
    line_number = rand(1..LINES)
    if classified_tweets.has_key?(line_number-1)
      redo
    else
      tweet = tweets[line_number-1]
      classified_tweets[line_number-1] = nil
    end
  end

  decoded_tweet = JSON.parse(tweet)
  puts "Random number   -   #{line_number-1}"
  print "Tweet #{i}\n#{decoded_tweet['text']}\n"

  # Open the web pages at the tweet in the google chrome browser
  if (webpages = decoded_tweet['text'].scan(/https?:\/\/[a-z0-9A-Z\/\.]*[a-z0-9A-Z]/))
    webpages.each do |webpage|
      `google-chrome #{webpage}`
    end
  end

  option = STDIN.gets.chomp

  case option
    when "1"
      files[:cybersecurity_news].write("#{tweet}")
    when "2"
      files[:potential_cybersecurity_threats].write("#{tweet}")
    when "3"
      files[:cybersecurity_threats].write("#{tweet}")
    when "4"
      files[:cybersecurity_threats_general_security_news].write("#{tweet}")      
    when "5"
      files[:general_security_news_cybersecurity_news].write("#{tweet}")
    when "6"
      files[:general_security_news].write("#{tweet}")
    when "7"
      files[:spam].write("#{tweet}")
    when "exit"
      break
    else
      puts "You typed a wrong option. Type a valid option."
      print instructions
      redo
  end

  i += 1
  line = nil
  files[:classified_tweets_last_run].write("#{tweet}")
  tweets[line_number-1] = ''
end

files.each_value {|file| file.close}

File.open(ARGV[0], "w") do |file|
  tweets.each do |line|
    file.print line
  end
end

# Old way to remove the classified tweets from the dataset (population of data or huge sample) file
# `touch #{FILE_FOLDER}/remanining_tweets_to_classify.json`
# system("diff --unchanged-line-format="" #{ARGV[0]} #{classes[:classified_tweets]} > #{FILE_FOLDER}/remanining_tweets_to_classify.json")
# `rm #{ARGV[0]}`
# `mv #{FILE_FOLDER}/remanining_tweets_to_classify.json #{ARGV[0]}`



# When you want to encode json document to string do JSON.dump(doc)
# The document will look like the original one except for the '/' that won't be scaped by '\'
# just write that in the file and it will be ok: JSON.dump(var.gsub(///, '\/'))